#!/usr/bin/env node
function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var sade = _interopDefault(require('sade'));
var karmatic = _interopDefault(require('./index.js'));
var fs = _interopDefault(require('fs'));
var path = _interopDefault(require('path'));
var chalk = _interopDefault(require('chalk'));

var ref = process.stdout;
var write = ref.write;

process.stdout.write = function (msg) {
  // Strip + prettify console forwarded output:
  var matches = msg.match(/^LOG ([A-Z]+): ([\s\S]*)$/);

  if (matches) {
    msg = chalk.bgBlueBright.white(' ' + matches[1] + ': ') + ' ' + chalk.blue(matches[2]);
  } // Strip browser prefix from the output since there's only one:


  if (msg.match(/^[\n\s]*HeadlessChrome/)) {
    var color = /\bSUCCESS\b/.test(msg) ? 'greenBright' : 'magenta';
    msg = chalk[color](msg.replace(/^[\n\s]*.*?: /g, ''));
  } // Ignore total output since we only have one browser:
  // eslint-disable-next-line no-control-regex


  if (msg.match(/\u001b\[32mTOTAL: /)) { return; }
  return write.call(process.stdout, msg);
};

var firstFile;
function cleanStack(str, cwd) {
  if ( cwd === void 0 ) cwd = process.cwd();

  str = str.replace(/^[\s\S]+\n\n([A-Za-z]*Error: )/g, '$1');
  firstFile = null;
  var clean = str.replace(new RegExp(("( |\\()(https?:\\/\\/localhost:\\d+\\/base\\/|" + (cwd.replace(/([\\/[\]()*+$!^.,?])/g, '\\$1')) + "\\/*)?([^\\s():?]*?)(?:\\?[a-zA-Z0-9]+?)?(:\\d+(?::\\d+)?)"), 'g'), replacer);

  if (firstFile != null) {
    var filename = firstFile[0];
    var line = firstFile[1];
    var char = firstFile[2];

    if (line) {
      var read;

      try {
        read = fs.readFileSync(path.resolve(cwd, filename), 'utf8');
      } catch (e) {}

      if (read) {
        var start = Math.max(0, char - 40);
        var startLine = Math.max(0, line - 3);
        read = read.split('\n');
        clean = clean.replace(/\n +/g, '\n  ');

        if (line < read.length) {
          clean += '\n\n' + chalk.white(highlight(outdent([read.slice(startLine, line).join('\n'), read[line].substr(start, char + 30), new Array(char).join('-') + '^', read.slice(line + 1, line + 4).join('\n')].join('\n'), '    ', process.stdout.columns - 10), line - startLine, 2));
        }
      }
    }
  }

  return chalk.red(clean);
}

function replacer(str, before, root, filename, position) {
  if (firstFile == null) {
    var ref = position.match(/\d+/g) || [];
    var line = ref[0];
    var char = ref[1];
    firstFile = [filename, line - 1, char | 0];
  }

  return before + chalk.blue('./' + filename + chalk.dim(position));
}

function highlight(text, line, count) {
  var lines = text.split('\n');
  return chalk.dim(lines.slice(0, line).join('\n')) + '\n' + lines.slice(line, line + count).join('\n') + '\n' + chalk.dim(lines.slice(line + count).join('\n'));
}

function outdent(str, prefix, width) {
  if ( prefix === void 0 ) prefix = '';
  if ( width === void 0 ) width = 80;

  str = str.replace(/(^\n+|\n+$)/g, '').replace(/\t/, '    ');
  var indents = str.match(/^[ -]+/gm) || [];
  var minLength = indents.reduce(function (indent, value) { return Math.min(indent, value.length); }, indents[0] ? indents[0].length : 0);
  str = str.replace(/^[ -]+/gm, function (str) { return prefix + str.substring(minLength); });
  str = str.replace(/^.*$/gm, function (str) { return str.substring(0, width); });
  return str;
}

var ref$1 = require('../package.json');
var version = ref$1.version;

var toArray = function (val) { return typeof val === 'string' ? val.split(/\s*,\s*/) : val == null ? [] : [].concat(val); };

var prog = sade('karmatic');
prog.version(version).option('--files', 'Minimatch pattern for test files').option('--headless', 'Run using Chrome Headless', true).option('--coverage', 'Report code coverage of tests', true).option('--downlevel', 'Downlevel syntax to ES5');
prog.command('run [...files]', '', {
  default: true
}).describe('Run tests once and exit').action(run);
prog.command('watch [...files]').describe('Run tests on any change').action(function (str, opts) { return run(str, opts, true); });
prog.command('debug [...files]').describe('Open a headful Puppeteer instance for debugging your tests').option('--headless', 'Run using Chrome Headless', false) // Override default to false
.option('--browsers', 'Run in specific browsers', null).option('--coverage', 'Report code coverage of tests', false) // Override default to false
.action(function (str, opts) { return run(str, opts, true); });
prog.parse(process.argv);

function run(str, opts, isWatch) {
  opts.watch = !!isWatch;
  opts.files = toArray(str || opts.files).concat(opts._);
  var b = opts.browsers || opts.browser;
  opts.browsers = b ? toArray(b) : null;
  karmatic(opts).then(function (output) {
    if (output != null) { process.stdout.write(output + '\n'); }
    if (!opts.watch) { process.exit(0); }
  }).catch(function (err) {
    if (!(typeof err.code === 'number' && err.code >= 0 && err.code < 10)) {
      process.stderr.write(chalk.red(cleanStack(err && (err.stack || err.message) || err)) + '\n');
    }

    process.exit(err.code || 1);
  });
}
//# sourceMappingURL=cli.js.map
