function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var puppeteer = _interopDefault(require('puppeteer'));
var delve = _interopDefault(require('dlv'));
var fs = _interopDefault(require('fs'));
var path = _interopDefault(require('path'));
var chalk = _interopDefault(require('chalk'));

function fileExists(file) {
  try {
    return fs.statSync(file).isFile();
  } catch (e) {}

  return false;
}
function readFile(file) {
  try {
    return fs.readFileSync(file, 'utf8');
  } catch (e) {}
}
function readDir(file) {
  try {
    return fs.readdirSync(file);
  } catch (e) {}
}
function tryRequire(file) {
  if (fileExists(file)) { return require(file); }
}
function dedupe(value, index, arr) {
  return arr.indexOf(value) === index;
}
var firstFile;
function cleanStack(str, cwd) {
  if ( cwd === void 0 ) cwd = process.cwd();

  str = str.replace(/^[\s\S]+\n\n([A-Za-z]*Error: )/g, '$1');
  firstFile = null;
  var clean = str.replace(new RegExp(("( |\\()(https?:\\/\\/localhost:\\d+\\/base\\/|" + (cwd.replace(/([\\/[\]()*+$!^.,?])/g, '\\$1')) + "\\/*)?([^\\s():?]*?)(?:\\?[a-zA-Z0-9]+?)?(:\\d+(?::\\d+)?)"), 'g'), replacer);

  if (firstFile != null) {
    var filename = firstFile[0];
    var line = firstFile[1];
    var char = firstFile[2];

    if (line) {
      var read;

      try {
        read = fs.readFileSync(path.resolve(cwd, filename), 'utf8');
      } catch (e) {}

      if (read) {
        var start = Math.max(0, char - 40);
        var startLine = Math.max(0, line - 3);
        read = read.split('\n');
        clean = clean.replace(/\n +/g, '\n  ');

        if (line < read.length) {
          clean += '\n\n' + chalk.white(highlight(outdent([read.slice(startLine, line).join('\n'), read[line].substr(start, char + 30), new Array(char).join('-') + '^', read.slice(line + 1, line + 4).join('\n')].join('\n'), '    ', process.stdout.columns - 10), line - startLine, 2));
        }
      }
    }
  }

  return chalk.red(clean);
}

function replacer(str, before, root, filename, position) {
  if (firstFile == null) {
    var ref = position.match(/\d+/g) || [];
    var line = ref[0];
    var char = ref[1];
    firstFile = [filename, line - 1, char | 0];
  }

  return before + chalk.blue('./' + filename + chalk.dim(position));
}

function highlight(text, line, count) {
  var lines = text.split('\n');
  return chalk.dim(lines.slice(0, line).join('\n')) + '\n' + lines.slice(line, line + count).join('\n') + '\n' + chalk.dim(lines.slice(line + count).join('\n'));
}

function outdent(str, prefix, width) {
  if ( prefix === void 0 ) prefix = '';
  if ( width === void 0 ) width = 80;

  str = str.replace(/(^\n+|\n+$)/g, '').replace(/\t/, '    ');
  var indents = str.match(/^[ -]+/gm) || [];
  var minLength = indents.reduce(function (indent, value) { return Math.min(indent, value.length); }, indents[0] ? indents[0].length : 0);
  str = str.replace(/^[ -]+/gm, function (str) { return prefix + str.substring(minLength); });
  str = str.replace(/^.*$/gm, function (str) { return str.substring(0, width); });
  return str;
}

function babelLoader(options) {
  return {
    test: /\.jsx?$/,
    exclude: /node_modules/,
    loader: require.resolve('babel-loader'),
    query: {
      presets: [[require.resolve('@babel/preset-env'), {
        targets: {
          browsers: ['last 2 Chrome versions', 'last 2 Firefox versions', (options.downlevel || options.browsers && String(options.browsers).match(/(\bie(\b|\d)|internet.explorer)/gi)) && 'ie>=9'].filter(Boolean)
        },
        corejs: 2,
        useBuiltIns: 'usage',
        modules: false,
        loose: true
      }]],
      plugins: [[require.resolve('@babel/plugin-proposal-object-rest-spread'), {
        loose: true,
        useBuiltIns: true
      }], [require.resolve('@babel/plugin-transform-react-jsx'), {
        pragma: options.pragma || 'h'
      }]].concat(options.coverage ? [require.resolve('babel-plugin-istanbul')] : [])
    }
  };
}

function cssLoader(options) {
  return {
    test: /\.css$/,
    loader: 'style-loader!css-loader'
  };
}

var WEBPACK_VERSION = String(require('webpack').version || '3.0.0');
var WEBPACK_MAJOR = parseInt(WEBPACK_VERSION.split('.')[0], 10);
/**
 * @param {Object} options
 * @param {Array} options.files - Test files to run
 * @param {Array} [options.browsers] - Custom list of browsers to run in
 * @param {Boolean} [options.headless=false] - Run in Headless Chrome?
 * @param {Boolean} [options.watch=false] - Start a continuous test server and retest when files change
 * @param {Boolean} [options.coverage=false] - Instrument and collect code coverage statistics
 * @param {Object} [options.webpackConfig] - Custom webpack configuration
 * @param {Boolean} [options.downlevel=false] - Downlevel/transpile syntax to ES5
 */

function configure(options) {
  var ref, obj, obj$1, obj$2;

  var cwd = process.cwd(),
      res = function (file) { return path.resolve(cwd, file); };

  var files = options.files.filter(Boolean);
  if (!files.length) { files = ['**/{*.test.js,*_test.js}']; }
  process.env.CHROME_BIN = puppeteer.executablePath();
  var gitignore = (readFile(path.resolve(cwd, '.gitignore')) || '').replace(/(^\s*|\s*$|#.*$)/g, '').split('\n').filter(Boolean);
  var repoRoot = (readDir(cwd) || []).filter(function (c) { return c[0] !== '.' && c !== 'node_modules' && gitignore.indexOf(c) === -1; });
  var rootFiles = '{' + repoRoot.join(',') + '}';
  var PLUGINS = ['karma-chrome-launcher', 'karma-jasmine', 'karma-spec-reporter', 'karma-sourcemap-loader', 'karma-webpack'].concat(options.coverage ? 'karma-coverage' : []); // Custom launchers to be injected:

  var launchers = {};
  var useSauceLabs = false;
  var browsers;

  if (options.browsers) {
    browsers = options.browsers.map(function (browser) {
      if (/^chrome$/i.test(browser)) {
        return 'Chrome';
      }

      if (/^firefox$/i.test(browser)) {
        PLUGINS.push('karma-firefox-launcher');
        return 'Firefox';
      }

      if (/^sauce-/.test(browser)) {
        if (!useSauceLabs) {
          useSauceLabs = true;
          PLUGINS.push('karma-sauce-launcher');
        }

        var parts = browser.toLowerCase().split('-');
        var name = parts.join('_');
        launchers[name] = {
          base: 'SauceLabs',
          browserName: parts[1].replace(/^ie$/gi, 'Internet Explorer'),
          version: parts[2] || undefined,
          platform: parts[3] ? parts[3].replace(/^win(dows)?[ -]+/gi, 'Windows ').replace(/^(macos|mac ?os ?x|os ?x)[ -]+/gi, 'OS X ') : undefined
        };
        return name;
      }

      return browser;
    });
  } else {
    browsers = [options.headless === false ? 'KarmaticChrome' : 'KarmaticChromeHeadless'];
  }

  if (useSauceLabs) {
    var missing = ['SAUCE_USERNAME', 'SAUCE_ACCESS_KEY'].filter(function (x) { return !process.env[x]; })[0];

    if (missing) {
      throw '\n' + chalk.bold.bgRed.white('Error:') + ' Missing SauceLabs auth configuration.' + '\n  ' + chalk.white(("A SauceLabs browser was requested, but no " + (chalk.magentaBright(missing)) + " environment variable provided.")) + '\n  ' + chalk.white('Try prepending it to your test command:') + '  ' + chalk.greenBright(missing + '=... npm test') + '\n';
    }
  }

  var WEBPACK_CONFIGS = ['webpack.config.babel.js', 'webpack.config.js'];
  var webpackConfig = options.webpackConfig;
  var pkg = tryRequire(res('package.json'));

  if (pkg.scripts) {
    for (var i in pkg.scripts) {
      var script = pkg.scripts[i];

      if (/\bwebpack\b[^&|]*(-c|--config)\b/.test(script)) {
        var matches = script.match(/(?:-c|--config)\s+(?:([^\s])|(["'])(.*?)\2)/);
        var configFile = matches && (matches[1] || matches[2]);
        if (configFile) { WEBPACK_CONFIGS.push(configFile); }
      }
    }
  }

  if (!webpackConfig) {
    for (var i$1 = WEBPACK_CONFIGS.length; i$1--;) {
      webpackConfig = tryRequire(res(WEBPACK_CONFIGS[i$1]));
      if (webpackConfig) { break; }
    }
  }

  if (typeof webpackConfig === 'function') {
    webpackConfig = webpackConfig({
      karmatic: true
    }, {
      mode: 'development',
      karmatic: true
    });
  }

  webpackConfig = webpackConfig || {};
  var loaders = [].concat(delve(webpackConfig, 'module.loaders') || [], delve(webpackConfig, 'module.rules') || []);

  function evaluateCondition(condition, filename, expected) {
    if (typeof condition === 'function') {
      return condition(filename) == expected;
    } else if (condition instanceof RegExp) {
      return condition.test(filename) == expected;
    }

    if (Array.isArray(condition)) {
      for (var i = 0; i < condition.length; i++) {
        if (evaluateCondition(condition[i], filename)) { return expected; }
      }
    }

    return !expected;
  }

  function getLoader(predicate) {
    if (typeof predicate === 'string') {
      var filename = predicate;

      predicate = function (loader) {
        var test = loader.test;
        var include = loader.include;
        var exclude = loader.exclude;
        if (exclude && evaluateCondition(exclude, filename, false)) { return false; }
        if (include && !evaluateCondition(include, filename, true)) { return false; }
        if (test && evaluateCondition(test, filename, true)) { return true; }
        return false;
      };
    }

    for (var i = 0; i < loaders.length; i++) {
      if (predicate(loaders[i])) {
        return {
          index: i,
          loader: loaders[i]
        };
      }
    }

    return false;
  }

  function webpackProp(name, value) {
    var configured = delve(webpackConfig, name);

    if (Array.isArray(value)) {
      return value.concat(configured || []).filter(dedupe);
    }

    return Object.assign({}, configured || {}, value);
  }

  var generatedConfig = {
    basePath: cwd,
    plugins: PLUGINS.map(function (req) { return require.resolve(req); }),
    frameworks: ['jasmine'],
    reporters: ['spec'].concat(options.coverage ? 'coverage' : [], useSauceLabs ? 'saucelabs' : []),
    browsers: browsers,
    sauceLabs: {
      testName: pkg && pkg.name || undefined
    },
    customLaunchers: Object.assign({
      KarmaticChrome: {
        base: 'Chrome',
        flags: ['--no-sandbox']
      },
      KarmaticChromeHeadless: {
        base: 'ChromeHeadless',
        flags: ['--no-sandbox']
      }
    }, launchers),
    coverageReporter: {
      reporters: [{
        type: 'text-summary'
      }, {
        type: 'html'
      }, {
        type: 'lcovonly',
        subdir: '.',
        file: 'lcov.info'
      }]
    },

    formatError: function formatError(msg) {
      try {
        msg = JSON.parse(msg).message;
      } catch (e) {}

      return cleanStack(msg);
    },

    logLevel: 'ERROR',
    loggers: [{
      type: path.resolve(__dirname, 'appender.js')
    }],
    files: (ref = [// @TODO remove me
      // { pattern: moduleDir('babel-polyfill')+'/dist/polyfill.js', watched: false, included: true, served: true }
    ]).concat.apply(ref, files.map(function (pattern) {
      // Expand '**/xx' patterns but exempt node_modules and gitignored directories
      var matches = pattern.match(/^\*\*\/(.+)$/);
      if (!matches) { return {
        pattern: pattern,
        watched: true,
        served: true,
        included: true
      }; }
      return [{
        pattern: rootFiles + '/' + matches[0],
        watched: true,
        served: true,
        included: true
      }, {
        pattern: matches[1],
        watched: true,
        served: true,
        included: true
      }];
    })),
    preprocessors: ( obj = {}, obj[rootFiles + '/**/*'] = ['webpack'], obj[rootFiles] = ['webpack'], obj ),
    webpack: {
      devtool: 'cheap-module-eval-source-map',
      // devtool: 'module-source-map',
      mode: webpackConfig.mode || 'development',
      module: {
        // @TODO check webpack version and use loaders VS rules as the key here appropriately:
        rules: loaders.concat(!getLoader(function (rule) { return ((rule.use) + "," + (rule.loader)).match(/\bbabel-loader\b/); }) && babelLoader(options), !getLoader('foo.css') && cssLoader(options)).filter(Boolean)
      },
      resolve: webpackProp('resolve', {
        modules: webpackProp('resolve.modules', ['node_modules', path.resolve(__dirname, '../node_modules')]),
        alias: webpackProp('resolve.alias', ( obj$1 = {}, obj$1[pkg.name] = res('.'), obj$1.src = res('src'), obj$1 ))
      }),
      resolveLoader: webpackProp('resolveLoader', {
        modules: webpackProp('resolveLoader.modules', ['node_modules', path.resolve(__dirname, '../node_modules')]),
        alias: webpackProp('resolveLoader.alias', ( obj$2 = {}, obj$2[pkg.name] = res('.'), obj$2.src = res('src'), obj$2 ))
      }),
      plugins: (webpackConfig.plugins || []).filter(function (plugin) {
        var name = plugin && plugin.constructor.name;
        return /^\s*(UglifyJS|HTML|ExtractText|BabelMinify)(.*Webpack)?Plugin\s*$/gi.test(name);
      }),
      node: webpackProp('node', {})
    },
    webpackMiddleware: {
      noInfo: true
    },
    colors: true,
    client: {
      captureConsole: true,
      jasmine: {
        random: false
      }
    }
  };

  if (WEBPACK_MAJOR < 4) {
    delete generatedConfig.webpack.mode;
    var ref$1 = generatedConfig.webpack.module;
    var rules = ref$1.rules;
    delete generatedConfig.webpack.module.rules;
    generatedConfig.webpack.module.loaders = rules;
  }

  return generatedConfig;
}

module.exports = configure;
//# sourceMappingURL=configure.js.map
